import streamlit as st
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from prophet import Prophet
from datetime import datetime
import plotly.graph_objects as go

# Custom styling for Streamlit UI
st.markdown(
    """
    <style>
    .stApp {background-color: #e6f7e6; padding: 10px;}
    .header {text-align: center; padding: 20px; font-family: Arial, sans-serif; background-color: #d4a9e8; color: white; border-radius: 8px;}
    .header h1 {margin: 0; color: #003366;}
    .header p {color: #36454F; }
    .stTextInput > div > div > input {background-color: #d9e9f8; color: #333; border: 1px solid #a9cce8; border-radius: 5px; padding: 8px;}
    .stSelectbox > div > div {background-color: #d9e9f8; border: 1px solid #a9cce8; border-radius: 5px;}
    .stButton > button {background-color: #6c63ff; color: white; border: none; padding: 10px 20px; border-radius: 8px; cursor: pointer; font-size: 16px;}
    .stButton > button:hover {background-color: #5a52d6;}
    .footer {text-align: center; font-size: 12px; color: #234; padding: 10px; border-top: 1px solid #ccc; margin-top: 50px;}
    </style>

    <div class="header">
        <h1>Franchise Order Forecaster </h1>
        <p>Predicting demand quantities for optimized franchise operations</p>
    </div>
    """,
    unsafe_allow_html=True
)

# Get the current year
current_year = datetime.now().year

# Generate future SEASON options dynamically
future_years = [current_year + i for i in range(10)]  # Next 10 years
seasons = ["FALL", "HOLIDAY", "SPRING", "SUMMER"]
season_options = [f"{season}-{year}" for year in future_years for season in seasons]

st.header("Enter Franchise, Season, and SKU Details")

# Input fields (removed YEAR input)
franchise_id = st.text_input("FRANCHISE_ID", "")
season = st.selectbox("SEASON", season_options)
sku = st.text_input("SKU", "")

# Load data from multiple sheets
@st.cache_data  # Cache the data loading to speed up subsequent runs
def load_data():
    df = pd.read_excel("/Users/rkadiyala/Downloads/PREDICTION_POC_DATA_Updated.xlsx", sheet_name=None)  # Load all sheets
    # Extract specific sheets from the loaded data
    sales_history = df["sales_history"]
    sku_details = df["sku_details"]
    franchise_partner = df["franchise_partner"]
    sales_prediction = df["sales_prediction"]

    # Convert SEASON to datetime (extract year from SEASON)
    season_to_month = {
        'FALL': '09',  # Fall corresponds to September
        'HOLIDAY': '12',  # Holiday corresponds to December
        'SPRING': '03',  # Spring corresponds to March
        'SUMMER': '06'   # Summer corresponds to June
    }

    # Extract YEAR and construct proper datetime 'ds'
    sales_history['SEASON'] = sales_history['SEASON'].astype(str)
    sales_history['YEAR'] = sales_history['SEASON'].str.extract(r'(\d{4})').astype(int)  # Extract year from SEASON
    sales_history['MONTH'] = sales_history['SEASON'].str.extract(r'([A-Za-z]+)-')[0].map(season_to_month)  # Map season to month

    # Create the 'ds' column for Prophet, ensuring it gets the correct month and year
    sales_history['ds'] = pd.to_datetime(sales_history['YEAR'].astype(str) + '-' + sales_history['MONTH'] + '-01')

    return sales_history, sku_details, franchise_partner, sales_prediction


sales_history, sku_details, franchise_partner, sales_prediction = load_data()

# Interactive and responsive plot function
def plot_sales_history_interactive(df_filtered, selected_season):
    # Extend the data to include selected season
    future_season = selected_season.split('-')
    future_year = int(future_season[1])
    future_season_name = future_season[0]

    # Extract existing seasons and add the future one
    unique_seasons = list(df_filtered['SEASON'].unique())
    if selected_season not in unique_seasons:
        unique_seasons.append(selected_season)

    # Sort seasons in chronological order
    sorted_seasons = sorted(unique_seasons, key=lambda x: (int(x.split('-')[1]), x.split('-')[0]))

    # Prepare data for plotting
    df_filtered['SEASON_DISPLAY'] = df_filtered['SEASON']  # Use season-year format for x-axis
    full_plot_df = pd.DataFrame({
        'SEASON_DISPLAY': sorted_seasons,
        'QUANTITY_ORDERED': [None] * len(sorted_seasons)  # Placeholder for missing data
    })

    # Merge actual data with placeholders
    plot_df = pd.merge(full_plot_df, df_filtered[['SEASON_DISPLAY', 'QUANTITY_ORDERED']],
                       on='SEASON_DISPLAY', how='left', suffixes=('_placeholder', '_actual'))
    plot_df['QUANTITY_ORDERED'] = plot_df['QUANTITY_ORDERED_actual'].fillna(plot_df['QUANTITY_ORDERED_placeholder'])

    # Create Plotly figure
    fig = go.Figure()
    fig.add_trace(go.Scatter(
        x=plot_df['SEASON_DISPLAY'],
        y=plot_df['QUANTITY_ORDERED'],
        mode='lines+markers',
        line=dict(color='blue', width=2),
        marker=dict(size=8),
        name='Quantity Ordered'
    ))

    fig.update_layout(
        title=f"Sales History for SKU {sku}",
        xaxis_title='Season-Year',
        yaxis_title='Quantity Ordered',
        xaxis=dict(type='category', title_font=dict(size=14), tickangle=45),
        yaxis=dict(title_font=dict(size=14)),
        template='plotly_white'
    )
    st.plotly_chart(fig, use_container_width=True)


if st.button('Predict Quantity'):
    if not franchise_id or not season:
        st.error("Please fill all required fields (FRANCHISE_ID and SEASON).")
    else:
        try:
            # Extract the year from the SEASON input
            year = int(season.split('-')[1])  # Get year from selected season (e.g., "FALL-2024" -> 2024)

            # Filter data based on provided inputs
            df_filtered = sales_history[
                (sales_history['FRANCHISE_ID'] == int(franchise_id)) &
                (sales_history['SEASON'].str.contains(season.split('-')[0]))
                ]

            # If SKU is provided, filter further
            if sku:
                df_filtered = df_filtered[df_filtered['SKU'] == int(sku)]

            # Check if filtered dataframe has at least 2 non-null rows
            if df_filtered[['ds', 'QUANTITY_ORDERED']].dropna().shape[0] < 2:
                st.error(f"Not enough valid data for FRANCHISE_ID: {franchise_id}, SKU: {sku or 'ALL'} in {season}.")
            elif df_filtered.empty:
                st.error(f"No data found for FRANCHISE_ID: {franchise_id}, SKU: {sku or 'ALL'} in {season}.")
            else:
                # Prepare data for Prophet
                model = Prophet()
                model.fit(df_filtered[['ds', 'QUANTITY_ORDERED']].rename(columns={'ds': 'ds', 'QUANTITY_ORDERED': 'y'}))

                # Create future dataframe
                future = model.make_future_dataframe(periods=1, freq='Y')  # Predict for next year

                # Make prediction
                forecast = model.predict(future)
                prediction = forecast['yhat'].iloc[-1]  # Get the last prediction

                # Save predicted quantity to the database (Mock implementation here)
                st.success(f"Predicted Quantity for SKU {sku or 'ALL'} in {season}: {round(max(prediction, 0))}")
                st.info("Prediction has been saved to the database.")

                # Plot sales history graph
                graph_title = f"Sales History for SKU {sku}" if sku else f"Sales History for FRANCHISE_ID {franchise_id}"
                plot_sales_history_interactive(df_filtered, season)

                # Calculate model accuracy
                historical_forecast = model.predict(df_filtered[['ds']])
                df_filtered['yhat'] = historical_forecast['yhat']
                df_filtered['error'] = abs(df_filtered['QUANTITY_ORDERED'] - df_filtered['yhat'])
                df_filtered['percentage_error'] = (
                                                          df_filtered['error'] / df_filtered[
                                                      'QUANTITY_ORDERED'].replace(0, float('nan'))
                                                  ) * 100  # Avoid division by zero

                # Drop rows where percentage error could not be calculated
                valid_errors = df_filtered.dropna(subset=['percentage_error'])

                # Debugging: Check what data is being used to calculate accuracy
                st.write("Debugging: Filtered Data for Accuracy Calculation")
                st.write(valid_errors[['ds', 'QUANTITY_ORDERED', 'yhat', 'error', 'percentage_error']])

                if not valid_errors.empty:
                    mape = valid_errors['percentage_error'].mean()  # Mean Absolute Percentage Error
                    accuracy = 100 - mape
                    st.info(f"Model Accuracy: {round(accuracy, 2)}%")
                else:
                    st.warning("Model Accuracy could not be calculated due to insufficient valid data.")

        except ValueError as e:
            st.exception(e)  # Show detailed error message for debugging
        except Exception as e:
            st.exception(f"An error occurred: {e}")

# Correlation Heatmap
st.markdown("<hr>", unsafe_allow_html=True)  # Horizontal Line to separate heatmap
st.subheader("Sales History Correlation Heatmap")
numeric_df = sales_history.select_dtypes(include=['float64', 'int64'])
corr = numeric_df.corr()
plt.figure(figsize=(8, 6))  # Reduced size for better display in Streamlit
sns.heatmap(corr, cbar=True, square=True, annot=True, fmt='.2f', cmap='coolwarm')
st.pyplot(plt.gcf())

st.markdown('<div class="footer">Powered by GAP AI</div>', unsafe_allow_html=True)

