from datetime import datetime

import streamlit as st
import pandas as pd
import plotly.express as px
from sqlalchemy import create_engine
import psycopg2

st.markdown(
    """
    <style>
    .stApp {
        background-color: #e6f7e6; 
        padding: 10px;
    }
    .header {
        text-align: center; 
        padding: 10px; 
        font-family: Arial, sans-serif; 
        background: linear-gradient(to right, red, orange, yellow, green, blue, indigo, violet); /* Rainbow gradient */
        color: white; 
        border-radius: 8px;
    }
    .header h1 {
        margin: 0; 
        color: white; /* White text for good contrast */
        text-shadow: 1px 1px 2px black; /* Optional: Shadow for readability */
    }
    .header p {
        color: #f0f0f0; /* Light text color for subtitle */
    }

    <div class="header">
        <h1>Franchise Order Forecaster</h1>
        <p>Predicting demand quantities for optimized franchise operations</p>
    </div>
    

    /* Consistent input styles (number input, text input, and selectbox) */
    input[type="number"], input[type="text"], .stSelectbox > div {
        background-color: #d9e9f8; 
        color: #003366; /* Darker text color for better contrast */
        border: 1px solid #a9cce8; 
        border-radius: 5px; 
    }

    /* Ensure dropdown selected value is visible */
    .stSelectbox > div > div {
        background-color: #d9e9f8 !important; 
        color: #003366 !important; /* Dark text for dropdown value */
    }

    /* Target the selected value (specific to Streamlit’s dropdowns) */
    .stSelectbox > div > div > div {
        color: #003366 !important; /* Dark text for selected option */
    }

    /* Button styles */
    .stButton > button {
        background-color: #6c63ff; 
        color: white; 
        border: none; 
        padding: 10px 20px; 
        border-radius: 8px; 
        cursor: pointer; 
        font-size: 16px;
    }
    .stButton > button:hover {
        background-color: #5a52d6;
    }

    /* Footer styles */
    .footer {
        text-align: center; 
        font-size: 12px; 
        color: #234; 
        padding: 10px; 
        border-top: 1px solid #ccc; 
        margin-top: 50px;
    }
    </style>

    <div class="header">
        <h1>Franchise Order Forecaster</h1>
        <p>Predicting demand quantities for optimized franchise operations</p>
    </div>
    """,
    unsafe_allow_html=True
)



# Database connection function
def get_database_connection():
    try:
        connection = psycopg2.connect(
            database="postgres",
            user="postgres",
            password="Welcome@1",
            host="localhost",
            port="5432"
        )
        return connection
    except Exception as e:
        st.error(f"Database connection failed: {str(e)}")
        return None


# Function to fetch data from database
def fetch_sales_data(franchise_id, season, sku=None):
    conn = get_database_connection()
    if conn:
        try:
            query = """
                SELECT franchise_id, season, sku, prediction_quantity 
                FROM public.sales_prediction 
                WHERE franchise_id = %s AND season = %s
            """
            params = [franchise_id, season]

            if sku:
                query += " AND sku = %s"
                params.append(sku)

            df = pd.read_sql_query(query, conn, params=params)
            conn.close()
            return df
        except Exception as e:
            st.error(f"Error fetching data: {str(e)}")
            conn.close()
            return None
    return None


current_year = datetime.now().year
future_years = [current_year + i for i in range(10)]  # Next 10 years
seasons = ["FALL", "HOLIDAY", "SPRING", "SUMMER"]
season_options = [f"{season}-{year}" for year in future_years for season in seasons]


franchise_id = st.number_input("FRANCHISE ID", min_value=30101, value=30101)
season = st.selectbox("SEASON", season_options)
sku = st.text_input("SKU (optional)", "")

# Get Data button
if st.button("Get Data"):
    if not season:
        st.warning("Please enter a season")

    # Fetch data
    df = fetch_sales_data(franchise_id, season, sku if sku else None)

    if df is not None and not df.empty:
        # Display the raw data
        st.dataframe(df, use_container_width=True)

        fig_bar = px.bar(
            df,
            x='sku',
            y='prediction_quantity',
            title='Prediction Quantity by SKU',
            labels={'prediction_quantity': 'Prediction Quantity', 'sku': 'SKU'}
        )
        st.plotly_chart(fig_bar)


else:
    st.error("No data found for the given parameters")
