from datetime import datetime

import streamlit as st
import pandas as pd
import plotly.express as px
from sqlalchemy import create_engine
import psycopg2


# Page configuration
st.set_page_config(page_title="Franchise Data Visualization", layout="wide")


# Database connection function
def get_database_connection():
    try:
        connection = psycopg2.connect(
            database="postgres",
            user="postgres",
            password="Welcome@1",
            host="localhost",
            port="5432"
        )
        return connection
    except Exception as e:
        st.error(f"Database connection failed: {str(e)}")
        return None


# Function to fetch data from database
def fetch_sales_data(franchise_id, season, sku=None):
    conn = get_database_connection()
    if conn:
        try:
            query = """
                SELECT franchise_id, season, sku, prediction_quantity 
                FROM public.sales_prediction 
                WHERE franchise_id = %s AND season = %s
            """
            params = [franchise_id, season]

            if sku:
                query += " AND sku = %s"
                params.append(sku)

            df = pd.read_sql_query(query, conn, params=params)
            conn.close()
            return df
        except Exception as e:
            st.error(f"Error fetching data: {str(e)}")
            conn.close()
            return None
    return None

current_year = datetime.now().year
future_years = [current_year + i for i in range(10)]  # Next 10 years
seasons = ["FALL", "HOLIDAY", "SPRING", "SUMMER"]
season_options = [f"{season}-{year}" for year in future_years for season in seasons]

# Main app
def main():
    st.title("Franchise Order Forecaster")

    # Create three columns for inputs
    col1, col2, col3 = st.columns(3)

    with col1:
        franchise_id = st.number_input("Enter Franchise ID", min_value=30101, value=30105)

    with col2:
        # season = st.text_input("Enter Season")
        season = st.selectbox("SEASON", season_options)
    with col3:
        sku = st.text_input("Enter SKU (optional)", "")

    # Get Data button
    if st.button("Get Data"):
        if not season:
            st.warning("Please enter a season")
            return

        # Fetch data
        df = fetch_sales_data(franchise_id, season, sku if sku else None)

        if df is not None and not df.empty:
            # Display the raw data
            st.subheader("Raw Data")
            st.dataframe(df)

            

            col1, col2 = st.columns(2)

            with col1:
                # Bar chart for quantity by SKU
                fig_bar = px.bar(
                    df,
                    x='sku',
                    y='prediction_quantity',
                    title='Prediction Quantity by SKU',
                    labels={'prediction_quantity': 'Prediction Quantity', 'sku': 'SKU'}
                )
                st.plotly_chart(fig_bar)

            with col2:
                # Pie chart showing distribution of quantities
                fig_pie = px.pie(
                    df,
                    values='prediction_quantity',
                    names='sku',
                    title='Distribution of Quantities by SKU'
                )
                st.plotly_chart(fig_pie)

            # Summary statistics
            st.subheader("Summary Statistics")
            st.write(df.describe())

        else:
            st.error("No data found for the given parameters")


if __name__ == "__main__":
    main()